--[[ FriendlistHandler
-- FriendlistHandler organizes everything what you may do with the global variable "FriendlistData".
-- There are these fields:
--      * Friends     a list of friends, indexed by the ingame name
--      * numFriends  number of friends
--      * version     version of the structure of the table and its content
--]]

FriendlistHandler = { };

-- FriendlistData = { };


local rev = 20130317;

--[[ Initializes the variable FriendlistData if empty --]]
function FriendlistHandler:Init()
    if FriendlistData == nil then
        FriendlistData = {};
        FriendlistData.version = rev;
    end
    if FriendlistData.Friends == nil then
        FriendlistData.Friends = {};
    end

    if FriendlistData.numFriends == nil or not tonumber(FriendlistData.numFriends) or FriendlistData.numFriends < 0 then
        FriendlistData.numFriends = 0;
    else
        FriendlistData.numFriends = 0;
        for k, v in pairs(FriendlistData.Friends) do
            FriendlistData.numFriends = FriendlistData.numFriends + 1;
        end
    end
end

--[[ Checks if a friend is in the list --]]
function FriendlistHandler:ExistsEntry( friendID )
    if ( FriendlistData.Friends[friendID] ~= nil ) then
        return true;
    else
        return false;
    end
end

--[[ Creates an empty entry --]]
function FriendlistHandler:CreateEntry(friendID)
    self:Init();

    if ( FriendlistData.Friends[friendID] == nil ) then
        FriendlistData.Friends[friendID] = {};
        
        FriendlistData.numFriends = FriendlistData.numFriends + 1;
    end
end

--[[ Changes the key of an entry --]]
function FriendlistHandler:ChangeKey( oldFriendKey, newFriendKey )
	if ( FriendlistData.Friends[oldFriendKey] == nil ) then
		return;
	end
    
    if ( oldFriendKey == newFriendKey ) then
        return;
    end
	
	-- Check to see that someone with this name doesn't exist.
	if ( FriendlistHandler:ExistsEntry(newFriendKey) ) then
		-- If the indices are different, then they are two different individuals, old friend can't move in.
		-- if ( self:GetIndex(newFriendKey) ~= self:GetIndex(oldFriendKey) ) then
			FlagRSPStatic:PrintE(string.gsub(FRIENDLIST_LOCALE_NameExistsMsg,"%%n",newFriendKey));
			return;
		-- end
	end
	
	local oldFriend = FriendlistData.Friends[oldFriendKey];
	FriendlistData.Friends[newFriendKey] = oldFriend;
	FriendlistData.Friends[oldFriendKey] = nil;
end

--[[ Deletes an friend entry --]]
function FriendlistHandler:DeleteEntry( friendID )
    FriendlistData.Friends[friendID] = nil;
    FriendlistData.numFriends = FriendlistData.numFriends - 1;
end

--[[ Adds/Updates a friendlist entry field
-- local bool = FriendlistHandler:UpdateInfo( field, value, friendID, typ )
--    Returns true if successful, else false (never nil)
]]--
function FriendlistHandler:UpdateInfo( field, value, friendID, typ )
    if ( field == nil or strtrim(field) == "" ) then
        return false;
    end

    if ( value == nil ) then
        return false;
    end
    
    if ( typ == nil ) then
        typ = "string";
    end

    self:CreateEntry(friendID);

    local entry = FriendlistData.Friends[friendID];
    if ( typ == "integer" ) then
        entry[field] = tonumber(value) or 0;
    elseif ( typ == "string" ) then
        entry[field] = tostring(value) or "";
    elseif ( typ == "bool" ) then
        if ( value == true ) then
            entry[field] = true;
        else
            entry[field] = false;
        end
    else
        return false;
    end

    return true;
end

--[[ Gets the content of a field
-- local tableInfo = FriendlistHandler:GetInfo( friendID, field, lookAll )
-- Returns the requested field of friendID from your friendlist or empty string
]]--
function FriendlistHandler:GetInfo( field, friendID )
    if ( self:ExistsEntry(friendID) and FriendlistData.Friends[friendID][field] ~= nil ) then
        return FriendlistData.Friends[friendID][field];
    else
        return "";
    end
end

--[[ Removes an info field --]]
function FriendlistHandler:RemoveInfo( field, friendID )
    if ( self:ExistsEntry(friendID) and FriendlistData.Friends[friendID][field] ~= nil ) then
        FriendlistData.Friends[friendID][field] = nil;
    end
end

--[[ Checks wether a field exists --]]
function FriendlistHandler:ExistsField( field, friendID )
    if ( self:ExistsEntry(friendID) and FriendlistData.Friends[friendID][field] ~= nil ) then
        return true;
    else
        return false;
    end
end

--[[ Deletes ALL Friend entries. Use with care! --]]
function FriendlistHandler:PurgeAllFriends()
    self:Reset();
end


--[[ Resets the FriendlistData to a completely new one --]]
function FriendlistHandler:Reset()
    FriendlistData = nil;

    self:Init();
end

--[[ Returns the version of FriendlistHandler --]]
function FriendlistHandler:GetVersion()
    return tonumber(rev) or 0;
end

function FriendlistHandler:UpdateVersion()
    FriendlistData.version = rev;
end

--[[ Returns the number of friends
-- Maintained internally! Fix with FriendlistHandler:Init()
--]]
function FriendlistHandler:GetNumFriends()
    return tonumber(FriendlistData.numFriends) or 0;
end

--[[ Returns the original friend list (no backup copy) --]]
function FriendlistHandler:GetFriends( )
    return FriendlistData.Friends;
end

--[[ Returns general statistics of the FriendlistData
--      Returns
--          numFriends  the known number of friends
--          numEntries  the actual number of friends
--          version     the actual version of the FriendlistData
--]]
function FriendlistHandler:GetStatistics()
    local numFriends = FriendlistData.numFriends;
    local version = FriendlistData.version;
    local numEntries = 0;

    for k, v in pairs(FriendlistData.Friends) do
        numEntries = numEntries + 1;
    end

    return numFriends, numEntries, version;
end
